#include "Arduino.h"
#include "stm32f446xx.h"
#include <SimpleFOC.h>
#include <STM32_CAN.h>
#include <AS5045.h>
#include <DRV8313.h>
#include <cstring>
#include <iostream>
#include <iterator>
#include "common/base_classes/FOCMotor.h"
#include "hal_conf_extra.h"
#include "wiring_analog.h"
#include "wiring_constants.h"
// clang-format on
#include "hal_conf_extra.h"
#include "reg_cah.h"
#include "flash.h"
#include "config.h"
#include "process_can.h"


void SysTick_Handler(void) {
  HAL_IncTick();
}

STM32_CAN Can(CAN2, DEF);
/*      for FLASH         */
uint32_t flash_flag;
uint8_t flag_can = 0;
uint32_t flash_error;
FLASH_EraseInitTypeDef pEraseInit;
uint32_t SectorError;
uint32_t timeout;

/* bool for test CAN    */
volatile bool CAN_GET = false;

volatile float kt = 0.1;  // Torque calculation constant
FLASH_RECORD* flash_rec;


SPIClass spi;
MagneticSensorAS5045 encoder(AS5045_CS, AS5045_MOSI, AS5045_MISO, AS5045_SCLK);

BLDCMotor motor(POLE_PAIRS);

DRV8313Driver driver(TIM1_CH1, TIM1_CH2, TIM1_CH3, EN_W_GATE_DRIVER,
                     EN_U_GATE_DRIVER, EN_V_GATE_DRIVER, SLEEP_DRIVER,
                     RESET_DRIVER, FAULT_DRIVER);
LowsideCurrentSense current_sense(0.01, 10.0, CURRENT_SENSOR_1,
                                  CURRENT_SENSOR_2, CURRENT_SENSOR_3);

// Commander command(Serial);

MotorControlInputs motor_control_inputs;

 volatile uint16_t msg_id;
 volatile uint16_t id_x;
 volatile uint8_t msg_ch;
 volatile uint8_t crc_h;
 volatile uint8_t crc_l;

volatile bool rx_can = false;
CAN_message_t msg_g;

  void setup(){
  SCB->VTOR = (volatile uint32_t)0x08008004;
  Serial.setRx(HARDWARE_SERIAL_RX_PIN);
  Serial.setTx(HARDWARE_SERIAL_TX_PIN);
  Serial.begin(115200);

  pinMode(PC11, OUTPUT);
  pinMode(PC10,OUTPUT);
  GPIOC->ODR &= ~GPIO_ODR_OD10;
  // Can.enableMBInterrupts();
  Can.begin();
  Can.setBaudRate(1000000);

  flash_rec = load_params();    //for update write_ptr
  if(flash_rec[firmw].value == FIRMWARE_FLAG) NVIC_SystemReset();  //if in flash go to the bootloader
  
  // Initialize FOC system
  setup_foc(&encoder, &motor, &driver, &current_sense,flash_rec);

  // Default motor configuration
  GPIOC->ODR |= GPIO_ODR_OD11;  //set LED
  motor.torque_controller = TorqueControlType::foc_current;
  motor.controller = MotionControlType::torque;
  __enable_irq();
  
}

bool is_can_busy() {
  return (CAN2->TSR & (CAN_TSR_TME0 | CAN_TSR_TME1 | CAN_TSR_TME2)) == 0;
}

void wait_for_can_tx_complete() {
    uint32_t start_time = HAL_GetTick();

    while (!(CAN2->TSR & (CAN_TSR_TME0 | CAN_TSR_TME1 | CAN_TSR_TME2))) {
        if (HAL_GetTick() - start_time > 500) break;
    }
    CAN2->TSR |= CAN_TSR_ABRQ0 | CAN_TSR_ABRQ1 | CAN_TSR_ABRQ2;
}

bool can_mailbox_free(uint8_t mailbox) {
  return (CAN2->TSR & (CAN_TSR_TME0 << mailbox)) != 0;
}

void clear_all_mailboxes() {
    CAN2->TSR |= CAN_TSR_ABRQ0 | CAN_TSR_ABRQ1 | CAN_TSR_ABRQ2;
    while ((CAN2->TSR & (CAN_TSR_ABRQ0 | CAN_TSR_ABRQ1 | CAN_TSR_ABRQ2)) != 0);
}
int can_err = 0;
void loop() {
  // Process incoming CAN messages
   if(rx_can){
    CAN_WAIT();
    clear_all_mailboxes();
    can_err = listen_can(msg_g);
    if(can_err == -1)
      HAL_Delay(1000);
    rx_can = false;
    CAN_STR();
}
  if(can_mailbox_free(0))
    send_velocity();
  while(!is_can_busy);
  if(can_mailbox_free(1))
    send_angle();
  while(!is_can_busy);
  __enable_irq();
  foc_step(&motor);
}

